#include "File.h"


namespace _FilePrivate
{
	void FileData::UnRef()
	{
		if (--refCount == 0)
		{
			if (flags & Flag_Opened)
			{
				close(hFile);
			}
			free((void*)this);
		}
	}
}

_FilePrivate::FileData TFile::_nullData = { 0, 1, 0 };


static inline bool _FileOpen(_FilePrivate::FileData*& res, const char* name, int openMode)
{
	if (openMode == 0) openMode = TFile::Mode_Read;

	int open_flags = 0;

	if (openMode & TFile::Mode_Read)     open_flags = O_RDONLY;
	if (openMode & TFile::Mode_Write)    open_flags = O_WRONLY;
	if (((openMode & (TFile::Mode_Read | TFile::Mode_Write)) == (TFile::Mode_Read | TFile::Mode_Write))) open_flags = O_RDWR;

	if (openMode & TFile::Mode_Create)   open_flags |= O_CREAT;
	//open_flags |= O_LARGEFILE;

	res = reinterpret_cast<_FilePrivate::FileData*>(malloc(sizeof(_FilePrivate::FileData)));
	if (!res) return false;

	res->refCount = 1;

	res->hFile = open(name, open_flags);

	if(res->hFile == -1)
	{
		free((void*)res);
		return false;
	}
	
	res->flags |= _FilePrivate::FileData::Flag_Opened;
	if (openMode & TFile::Mode_Read)  res->flags |= _FilePrivate::FileData::Flag_CanRead;
	if (openMode & TFile::Mode_Write) res->flags |= _FilePrivate::FileData::Flag_CanWrite;

	return true;
}


TFile::TFile()
{
	_data = _nullData.Ref();
}

TFile::TFile(const char* name, int openMode)
{
	if (!_FileOpen(_data, name, openMode))
	{
		_data = _nullData.Ref();
	}
}

TFile::~TFile()
{
	_data->UnRef();
}

bool TFile::Open(const char* name, int openMode)
{
	_data->UnRef();

	if (!_FileOpen(_data, name, openMode))
	{
		_data = _nullData.Ref();
		return false;
	}
	return true;
}

void TFile::Close()
{
	_data->UnRef();
	_data = _nullData.Ref();
}

ssize_t TFile::Read(void* buffer,size_t size)
{
	return ::read(_data->hFile, buffer, size);
	
}

ssize_t TFile::Write(const void* buffer, size_t size)
{
	return ::write(_data->hFile, buffer, size);
}

off_t TFile::SetPosition(off_t position)
{
	return ::lseek(_data->hFile, position, SEEK_SET);
}

TFile& TFile::operator=(const TFile& f)
{
	_FilePrivate::FileData* old = _data;
	_data = f._data->Ref();
	old->UnRef();
	return *this;
}
