#ifndef __FILE_H_INCLUDED
#define __FILE_H_INCLUDED

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>

namespace _FilePrivate
{
	struct FileData
	{
		int hFile;
		int refCount;
		enum
		{
			Flag_CanRead   = 1 << 0,
			Flag_CanWrite  = 1 << 1,
			Flag_Opened    = 1 << 3
		};
		int flags;

		FileData* Ref() { ++refCount; return this; }
		void UnRef();
	};
}


class TFile
{
	_FilePrivate::FileData* _data;
	static _FilePrivate::FileData _nullData;
public:
	enum
	{
		Mode_Read   = 1 << 0,
		Mode_Write  = 1 << 1,
		Mode_Create = 1 << 2,
		Mode_DeleteOnClose = 1 << 3
	};

	TFile();
	TFile(const char* name, int openMode = Mode_Read);
	~TFile();

	bool Open(const char* name, int openMode = Mode_Read);
	void Close();

	inline bool IsOpen()   const { return (_data->flags & _FilePrivate::FileData::Flag_Opened) == _FilePrivate::FileData::Flag_Opened; }
	inline bool CanRead()  const { return (_data->flags & _FilePrivate::FileData::Flag_CanRead) == _FilePrivate::FileData::Flag_CanRead; }
	inline bool CanWrite() const { return (_data->flags & _FilePrivate::FileData::Flag_CanWrite) == _FilePrivate::FileData::Flag_CanWrite; }

	ssize_t Read(void* buffer, size_t size);
	ssize_t Write(const void* buffer, size_t size);

	off_t SetPosition(off_t position);

	int Handle() { return _data->hFile; }

	TFile& operator=(const TFile& f);
};

#endif // __FILE_H_INCLUDED
