#include "IsoPrimaryDescriptor.h"
#include <string.h>

static void ReadString(std::string& target, const uint8_t* source, size_t max)
{
	size_t len = max;
	while (len > 0 && (source[len-1] == 0 || source[len - 1] == ' '))
	{
		--len;
	}
	
	target = std::string((const char*)source, len);
}

void IsoPrimaryDescriptor::load(const uint8_t* sector)
{
	ReadString(SystemID, sector + 8, 32);
	ReadString(VolumeID, sector + 40, 32);
	#if defined(LITTLE_ENDIAN)
	NumberOfSectors = *(uint32_t*)(sector + 80);
	VolumeSetSize = *(uint16_t*)(sector + 120);
	VolumeSequenceNumber = *(uint16_t*)(sector + 124);
	SectorSize = *(uint16_t*)(sector + 128);
	PathTableSize = *(uint32_t*)(sector + 132);
	FirstPathTableSector = *(uint32_t*)(sector + 140);
	SecondPathTableSector = *(uint32_t*)(sector + 144);
	#else
	NumberOfSectors = *(uint32_t*)(sector + 84);
	VolumeSetSize = *(uint16_t*)(sector + 122);
	VolumeSequenceNumber = *(uint16_t*)(sector + 126);
	SectorSize = *(uint16_t*)(sector + 130);
	PathTableSize = *(uint32_t*)(sector + 136);
	FirstPathTableSector = *(uint32_t*)(sector + 148);
	SecondPathTableSector = *(uint32_t*)(sector + 152);
	#endif
	memcpy(RootDirectoryRecord, sector + 156, 34);
	ReadString(VolumeSetID,   sector + 190, 128);
	ReadString(PublisherID,   sector + 318, 128);
	ReadString(PreparerID,    sector + 446, 128);
	ReadString(ApplicationID, sector + 574, 128);
}

PathTableEntry::PathTableEntry()
{
	
}

PathTableEntry::~PathTableEntry()
{
	for (std::vector<PathTableEntry*>::size_type i = 0; i < children.size(); ++i)
	{
		delete children[i];
	}
}

PathTableEntry* PathTableEntry::load(uint8_t*& data, uint32_t& remaining)
{
	if (remaining < ((uint32_t)8 + (uint32_t)data[0] + ((uint32_t)(data[0] & 1))))
	{
		remaining = 0;
		return 0;
	}

	PathTableEntry* e = new PathTableEntry();

	e->NameSize = data[0];
	e->ExtendedAttributeSectors = data[1];
	e->FistDirectorySector = *(uint32_t*)(data + 2);
	e->ParentDirectoryRecord = *(uint32_t*)(data + 6);
	ReadString(e->Name, data+8, e->NameSize);
	remaining -= 8 + e->NameSize + (e->NameSize & 1);
	data += (size_t)8 + e->NameSize + (e->NameSize & 1);
	return e;
}

DirectoryRecord::DirectoryRecord()
{}

DirectoryRecord::~DirectoryRecord()
{
	for (std::vector<DirectoryRecord*>::size_type i = 0; i < children.size(); ++i)
	{
		delete children[i];
	}
}

DirectoryRecord* DirectoryRecord::load(uint8_t*& data, uint32_t& remaining)
{
	if (remaining < 1)
	{
		return 0;
	}

	uint32_t r = data[0]; // record size

	// must be even
	if (((r & 1) != 0) || remaining < r)
	{
		return 0;
	}

	DirectoryRecord *record = new DirectoryRecord();
	record->ExtendedAttributeSize = data[1];
	#if defined(LITTLE_ENDIAN)
	record->Sector = *(uint32_t*)(data + 2);
	record->Size = *(uint32_t*)(data + 10);
	record->VolumeSequenceNumber = *(uint16_t*)(data + 28);
	#else
	record->Sector = *(uint32_t*)(data + 6);
	record->Size = *(uint32_t*)(data + 14);
	record->VolumeSequenceNumber = *(uint16_t*)(data + 30);
	#endif
	record->Flags = data[25];
	record->InterleavedFileUnitSize = data[26];
	record->InterleaveGapSize = data[27];

	uint32_t n = data[32]; // identifier length

	ReadString(record->Identifier, data + 33, n);

	uint32_t p; // padding
	if (n & 1)
	{
		p = 0;
	}
	else p = 1;

	data += r;
	remaining -= r;
	return record;
}
