#ifndef _ISO_PRIMARY_DESCRIPTOR
#define _ISO_PRIMARY_DESCRIPTOR

#include <sys/types.h>
#include <string>
#include <map>
#include <vector>

//#define LITTLE_ENDIAN

struct IsoPrimaryDescriptor
{
	std::string SystemID;
	std::string VolumeID;
	uint32_t    NumberOfSectors;
	uint16_t    VolumeSetSize;
	uint16_t    VolumeSequenceNumber;
	uint16_t    SectorSize;
	uint32_t    PathTableSize;
	uint32_t    FirstPathTableSector;
	uint32_t    SecondPathTableSector;
	uint8_t     RootDirectoryRecord[34];
	std::string VolumeSetID;
	std::string PublisherID;
	std::string PreparerID;
	std::string ApplicationID;

	void load(const uint8_t* sector);
};

struct DirectoryRecord
{
	uint8_t  ExtendedAttributeSize; // in sectors
	uint32_t Sector; // first sector of file
	uint32_t Size; // size of file (in bytes)
	uint8_t  Flags;
	uint8_t  InterleavedFileUnitSize;
	uint8_t  InterleaveGapSize;
	uint16_t VolumeSequenceNumber;
	std::string Identifier;

	std::vector<DirectoryRecord*> children;

	inline bool isNormal()    const { return !testFlag(0); }
	inline bool isHidden()    const { return testFlag(0); }
	inline bool isFile()      const { return !testFlag(1); }
	inline bool isDirectory() const { return testFlag(1); }
	inline bool associatedFile() const { return testFlag(2); }
	inline bool recordFormatSpecified() const { return testFlag(3); }
	inline bool permissionSpecified() const { return testFlag(4); }
	inline bool isFinalRecord() const { return !testFlag(7); }

	static DirectoryRecord* load(uint8_t*& data, uint32_t& remaining);
	~DirectoryRecord();

private:
	inline bool testFlag(uint8_t bit) const { return ((Flags & (1 << bit)) != 0); }
	DirectoryRecord();
};

struct PathTableEntry
{
	uint8_t NameSize;
	uint8_t ExtendedAttributeSectors;
	uint32_t FistDirectorySector;
	uint16_t ParentDirectoryRecord;
	std::string Name;

	std::vector<PathTableEntry*> children;

	~PathTableEntry();
	static PathTableEntry* load(uint8_t*& data, uint32_t& remaining);

private:
	PathTableEntry();
};

typedef std::map<uint16_t, PathTableEntry*> PathTableMap;

#define MAKE_LBA(sector, sector_size) \
	((off_t)(((off_t)(sector)) * ((off_t)(sector_size))))

#endif
